@@ -1,7 +1,7 @@
+import itertools
 import mmap
 import os
 import sys
-import itertools
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import suppress
 from io import BytesIO
@@ -12,11 +12,9 @@ import pyperclip
 from tqdm import tqdm
 
 from covert import lazyexec, passphrase, pubkey, tty, util
-from covert.archive import Archive
+from covert.archive import Archive, FileRecord
 from covert.blockstream import decrypt_file, encrypt_file
-
-ARMOR_MAX_SIZE = 32 << 20  # If output is a file (limit our memory usage)
-TTY_MAX_SIZE = 100 << 10  # If output is a tty (limit too lengthy spam)
+from covert.util import ARMOR_MAX_SIZE, TTY_MAX_SIZE
 
 
 def run_decryption(infile, args, auth):
@@ -146,7 +144,9 @@ def main_enc(args):
       try:
         recipients.append(pubkey.decode_pk(keystr))
       except ValueError as e:
-        if os.path.isfile(keystr):
+        if keystr.startswith("github:"):
+          raise ValueError(f"Unrecognized recipient string. Download a key from Github by -R {keystr}")
+        elif os.path.isfile(keystr):
           raise ValueError(f"Unrecognized recipient string. Use a keyfile by -R {keystr}")
         raise
     for fn in args.recipfiles:
@@ -203,52 +203,55 @@ def main_enc(args):
 
   a.nextfilecb = nextfile_callback
   # Main processing
-  with outf:
-    with tqdm(
-      total=a.total_size, delay=1.0, ncols=78, unit='B', unit_scale=True, bar_format="{l_bar}         {bar}{r_bar}"
-    ) as progress:
-      for block in encrypt_file((args.wideopen, pwhashes, recipients, signatures), a.encode, a):
-        progress.update(len(block))
-        outf.write(block)
-    # Pretty output printout
-    if sys.stderr.isatty():
-      # Print a list of files
-      lock = " 🔓 wide-open" if args.wideopen else " 🔒 covert"
-      methods = "  ".join(
-        [f"🔗 {r}" for r in recipients] + [f"🔑 {a}" for a in vispw] + (numpasswd - len(vispw)) * ["🔑 <pw>"]
-      )
-      methods += f' 🔷 {a.filehash[:12].hex()}'
-      for id in signatures:
-        methods += f"  🖋️ {id}"
-      if methods:
-        lock += f"    {methods}"
-      if args.outfile:
-        lock += f"  💾 {args.outfile}\n"
-      elif args.paste:
-        lock += f"  📋 copied\n"
-      out = f"\n\x1B[1m{lock}\x1B[0m\n"
-      sys.stderr.write(out)
+  with tqdm(
+    total=a.total_size, delay=1.0, ncols=78, unit='B', unit_scale=True, bar_format="{l_bar}         {bar}{r_bar}"
+  ) as progress:
+    for block in encrypt_file((args.wideopen, pwhashes, recipients, signatures), a.encode, a):
+      progress.update(len(block))
+      outf.write(block)
+  # Pretty output printout
+  if sys.stderr.isatty():
+    # Print a list of files
+    lock = " 🔓 wide-open" if args.wideopen else " 🔒 covert"
+    methods = "  ".join(
+      [f"🔗 {r}" for r in recipients] + [f"🔑 {a}" for a in vispw] + (numpasswd - len(vispw)) * ["🔑 <pw>"]
+    )
+    methods += f' 🔷 {a.filehash[:12].hex()}'
+    for id in signatures:
+      methods += f"  🖋️ {id}"
+    if methods:
+      lock += f"    {methods}"
+    if args.outfile:
+      lock += f"  💾 {args.outfile}\n"
+    elif args.paste:
+      lock += f"  📋 copied\n"
+    out = f"\n\x1B[1m{lock}\x1B[0m\n"
+    sys.stderr.write(out)
+    sys.stderr.flush()
+  if outf is not realoutf:
+    outf.seek(0)
+    data = outf.read()
+    data = util.armor_encode(data)
+  if outf is not realoutf:
+    if args.paste:
+      pyperclip.copy(f"```\n{data}\n```\n")
+      return
+    pretty = realoutf.isatty()
+    if pretty:
+      sys.stderr.write("\x1B[1;30m```\x1B[0;34m\n")
       sys.stderr.flush()
-    if outf is not realoutf:
-      outf.seek(0)
-      data = outf.read()
-      data = util.armor_encode(data)
-    if outf is not realoutf:
-      if args.paste:
-        pyperclip.copy(f"```\n{data}\n```\n")
-        return
-      with realoutf:
-        pretty = realoutf.isatty()
-        if pretty:
-          sys.stderr.write("\x1B[1;30m```\x1B[0;34m\n")
-          sys.stderr.flush()
-        try:
-          realoutf.write(f"{data}\n".encode())
-          realoutf.flush()
-        finally:
-          if pretty:
-            sys.stderr.write("\x1B[1;30m```\x1B[0m\n")
-            sys.stderr.flush()
+    try:
+      realoutf.write(f"{data}\n".encode())
+      realoutf.flush()
+    finally:
+      if pretty:
+        sys.stderr.write("\x1B[1;30m```\x1B[0m\n")
+        sys.stderr.flush()
+  # Not using `with outf` because closing stdout causes a lot of trouble and
+  # missing the close on a file when the CLI exits anyway is not dangerous.
+  # TODO: Delete the output file if any exception occurs.
+  if outf is not sys.stdout.buffer:
+    outf.close()
 
 
 def main_dec(args):
@@ -261,7 +264,14 @@ def main_dec(args):
   identities = list(sorted(identities, key=str))
   infile = open(args.files[0], "rb") if args.files else sys.stdin.buffer
   # If ASCII armored or TTY, read all input immediately (assumed to be short enough)
-  total_size = os.path.getsize(args.files[0]) if args.files else 0
+
+  # FIXME: For stdin the size is set to 50 so we try to read all of it (even if from a pipe),
+  # so that armoring can work. But this breaks pipe streaming of large files that cannot
+  # fit in RAM and tries to armor-decode very large files too. Needs to be fixed by
+  # attempting to read some to determine whether the input is small enough, and if not,
+  # use the covert.archive.CombinedIO object to consume the buffer already read and then
+  # resume streaming.
+  total_size = os.path.getsize(args.files[0]) if args.files else 50
   if infile.isatty():
     data = util.armor_decode(pyperclip.paste() if args.paste else tty.read_hidden("Encrypted message"))
     if not data:
@@ -269,7 +279,7 @@ def main_dec(args):
     infile = BytesIO(data)
     total_size = len(data)
     del data
-  elif 40 <= total_size <= ARMOR_MAX_SIZE:
+  elif 40 <= total_size <= 2 * ARMOR_MAX_SIZE:
     # Try reading the file as armored text rather than binary
     with infile:
       data = infile.read()
@@ -300,6 +310,56 @@ def main_dec(args):
     run_decryption(infile, args, authgen())
 
 
+def main_edit(args):
+  if len(args.files) != 1:
+    raise ValueError("Edit mode requires an encrypted archive filename (or '-' to use stdio).")
+  fname = args.files[0]
+  # Read all of input file (or stdin) to RAM
+  if fname is True:
+    data = sys.stdin.buffer.read()
+  else:
+    with open(fname, "rb") as f:
+      data = f.read()
+  try:
+    infile = BytesIO(util.armor_decode(data.decode()))
+    args.armor = True
+  except Exception:
+    infile = BytesIO(data)
+  # Decrypt everything to RAM
+  pwhash = passphrase.pwhash(passphrase.ask("Passphrase")[0])
+  a = Archive()
+  for data in a.decode(decrypt_file([pwhash], infile, a)):
+    if isinstance(data, dict): pass
+    elif isinstance(data, bool):
+      if data: a.curfile.data = bytearray()
+    else: a.curfile.data += data
+  # Edit the message (should be the first file)
+  if a.flist and a.flist[0].name is None:
+    a.flist[0].data = util.encode(tty.editor(a.flist[0].data.decode()))
+    a.flist[0].size = len(a.flist[0].data)
+  else:
+    data = util.encode(tty.editor())
+    a.flist.insert(0, FileRecord([len(data), None, {}]))
+    a.flist[0].data = data
+  # Reset archive for re-use in encryption
+  a.reset()
+  a.fds = [BytesIO(f.data) for f in a.flist]
+  a.random_padding()
+  # Encrypt in RAM...
+  out = bytearray()
+  for block in encrypt_file((False, [pwhash], [], []), a.encode, a):
+    out += block
+  # Preserve armoring if the input was armored
+  if args.armor:
+    out = f"{util.armor_encode(out)}\n".encode()
+  # Finally write output / replace the file
+  if fname is True:
+    sys.stdout.buffer.write(out)
+  else:
+    with open(fname, "wb") as f:
+      f.write(out)
+
+
 def main_benchmark(args):
 
   def noop_read(block):
@@ -316,7 +376,7 @@ def main_benchmark(args):
   ciphertext = mmap.mmap(-1, size)
   ciphertext[:] = bytes(size)
 
-  rounds = 5
+  rounds = 3
   enctotal = dectotal = 0
   for i in range(rounds):
     print("ENC", end="", flush=True)
