@@ -1,6 +1,6 @@
 import secrets
+import sys
 from contextlib import suppress
-from sys import argv
 
 import nacl.bindings as sodium
 from zxcvbn import zxcvbn
@@ -11,7 +11,7 @@ from covert.tty import fullscreen
 from covert.wordlist import words
 
 MINLEN = 8  # Bytes, not characters
-
+ARGON2_MEMLIMIT = 1 << 28  # Bytes (libsodium), as opposed to KiB (other libraries)
 
 def generate(n=4, sep=""):
   """Generate a password of random words without repeating any word."""
@@ -51,7 +51,7 @@ def _argon2(outlen, passwd, salt, ops):
     passwd=passwd,
     salt=salt,
     opslimit=ops,
-    memlimit=1 << 28,
+    memlimit=ARGON2_MEMLIMIT,
     alg=sodium.crypto_pwhash_ALG_ARGON2ID13,
   )
 
@@ -156,7 +156,7 @@ def ask(prompt, create=False):
 
 def pwhints(pwd: str):
   maxlen = 20  # zxcvbn gets slow with long passwords
-  z = zxcvbn(pwd[:maxlen], user_inputs=argv)
+  z = zxcvbn(pwd[:maxlen], user_inputs=sys.argv)
   fb = z["feedback"]
   warn = fb["warning"]
   sugg = fb["suggestions"]
